/**
  **************************************************************************
  * @file     at32_video_ev_sccb.c
  * @version  v2.0.2
  * @date     2022-09-15
  * @brief    at32_video_ev_sccb program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32_video_ev_sccb.h"

#define delay_cnt  5 //50

/**
  * @brief  initialize bf3901 sccb
  * @param  none
  * @retval none
  */
void bf3901_sccb_init(void)
{
  gpio_init_type  GPIO_InitStructure;
  crm_periph_clock_enable(BF3901_SCCB_GPIO_CLK, TRUE);

  gpio_default_para_init(&GPIO_InitStructure);
  GPIO_InitStructure.gpio_pins = BF3901_SCCB_PIN_SDA;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_INPUT;
  GPIO_InitStructure.gpio_pull = GPIO_PULL_UP;
  gpio_init(BF3901_SCCB_GPIO, &GPIO_InitStructure);

  GPIO_InitStructure.gpio_pins = BF3901_SCCB_PIN_SCL;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_OUTPUT;
  gpio_init(BF3901_SCCB_GPIO, &GPIO_InitStructure);

  gpio_bits_set(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SCL);

  GPIO_InitStructure.gpio_pins = GPIO_PINS_9;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_OUTPUT;
  gpio_init(GPIOB, &GPIO_InitStructure);

  gpio_bits_reset(GPIOB, GPIO_PINS_9);

  SCCB_SDA_OUT();
}

/**
  * @brief  sccb start signal
  * @param  none
  * @retval none
  */
void sccb_start(void)
{
  gpio_bits_set(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SDA);
  gpio_bits_set(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SCL);
  delay_us(delay_cnt);
  gpio_bits_reset(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SDA);
  delay_us(delay_cnt);
  gpio_bits_reset(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SCL);
}

/**
  * @brief  sccb stop signal
  * @param  none
  * @retval none
  */
void sccb_stop(void)
{
  gpio_bits_reset(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SDA);
  delay_us(delay_cnt);
  gpio_bits_set(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SCL);
  delay_us(delay_cnt);
  gpio_bits_set(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SDA);
  delay_us(delay_cnt);
}

/**
  * @brief  sccb no acknowledge
  * @param  none
  * @retval none
  */
void sccb_no_ack(void)
{
  delay_us(delay_cnt);
  gpio_bits_set(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SDA);
  gpio_bits_set(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SCL);
  delay_us(delay_cnt);
  gpio_bits_reset(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SCL);
  delay_us(delay_cnt);
  gpio_bits_reset(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SDA);
  delay_us(delay_cnt);
}

/**
  * @brief  sccb write byte
  * @param  dat: write data
  * @retval 0: success, 1 failure
  */
u8 sccb_wr_byte(u8 dat)
{
  u8 j, res;

  for(j = 0; j < 8; j++)
  {
    if(dat & 0x80)
    {
      gpio_bits_set(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SDA);
    }
    else
    {
      gpio_bits_reset(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SDA);
    }

    dat <<= 1;
    delay_us(delay_cnt);
    gpio_bits_set(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SCL);
    delay_us(delay_cnt);
    gpio_bits_reset(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SCL);
  }

  SCCB_SDA_IN();
  delay_us(delay_cnt);
  gpio_bits_set(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SCL);
  delay_us(delay_cnt);

  if(gpio_input_data_bit_read(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SDA))
  {
    res = 1;
  }
  else
  {
    res = 0;
  }

  gpio_bits_reset(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SCL);
  SCCB_SDA_OUT();
  return res;
}

/**
  * @brief  sccb read byte
  * @param  none
  * @retval read data
  */
u8 sccb_rd_byte(void)
{
  u8 temp = 0, j;
  SCCB_SDA_IN();

  for(j = 8; j > 0; j--)
  {
    delay_us(delay_cnt);
    gpio_bits_set(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SCL);
    temp = temp << 1;

    if(gpio_input_data_bit_read(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SDA))
    {
      temp++;
    }

    delay_us(delay_cnt);
    gpio_bits_reset(BF3901_SCCB_GPIO, BF3901_SCCB_PIN_SCL);
  }

  SCCB_SDA_OUT();
  return temp;
}

/**
  * @brief  sccb write register
  * @param  reg: write register
  * @param  dat: write data
  * @retval 0: success, 1 failure
  */
u8 sccb_wr_reg(u8 reg, u8 data)
{
  u8 res = 0;
  sccb_start();

  if(sccb_wr_byte(SCCB_ID))
  {
    res = 1;
  }

  delay_us(delay_cnt);

  if(sccb_wr_byte(reg))
  {
    res = 1;
  }

  delay_us(delay_cnt);

  if(sccb_wr_byte(data))
  {
    res = 1;
  }

  sccb_stop();
  return	res;
}

/**
  * @brief  sccb read register
  * @param  reg: read register
  * @retval read data
  */
u8 sccb_rd_reg(u8 reg)
{
  u8 val = 0;
  sccb_start();
  sccb_wr_byte(SCCB_ID);
  delay_us(delay_cnt);
  sccb_wr_byte(reg);
  delay_us(delay_cnt);
  sccb_stop();
  delay_us(delay_cnt);
  /* set register then read */
  sccb_start();
  sccb_wr_byte(SCCB_ID | 0x01);
  delay_us(delay_cnt);
  val = sccb_rd_byte();
  sccb_no_ack();
  sccb_stop();
  return val;
}
