/**
  **************************************************************************
  * @file     at32_video_ev_malloc.c
  * @version  v2.0.2
  * @date     2022-09-15
  * @brief    at32_video_ev_malloc program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32_video_ev_malloc.h"
#include "stdio.h"


/* memory pool */
__align(32) u8 mem1base[MEM1_MAX_SIZE];

/* memory manager map */
u16 mem1mapbase[MEM1_ALLOC_TABLE_SIZE];

/* memory parameters */
const u32 memtblsize[SRAMBANK] = {MEM1_ALLOC_TABLE_SIZE};	 //memory table size
const u32 memblksize[SRAMBANK] = {MEM1_BLOCK_SIZE};					//memory block size
const u32 memsize[SRAMBANK] = {MEM1_MAX_SIZE};							//memory max size

struct _m_mallco_dev mallco_dev =
{
  my_mem_init,
  my_mem_perused,
  mem1base,
  mem1mapbase,
  0,
};

/**
  * @brief  memory copy
  * @param  des: destination address
  * @param  src: source address
  * @param  n: number of copy
  * @retval none
  */
void mymemcpy(void *des, void *src, u32 n)
{
  u8 *xdes = des;
  u8 *xsrc = src;

  while(n--)
  {
    *xdes++ = *xsrc++;
  }
}

/**
  * @brief  memory set
  * @param  s: first address of memory
  * @param  c: set value
  * @param  count: set memory size(byte)
  * @retval none
  */
void mymemset(void *s, u8 c, u32 count)
{
  u8 *xs = s;

  while(count--)
  {
    *xs++ = c;
  }
}

/**
  * @brief  initialize memory
  * @param  memx: memory block
  * @retval none
  */
void my_mem_init(u8 memx)
{
  mymemset(mallco_dev.memmap[memx], 0, memtblsize[memx] * 2);
  mymemset(mallco_dev.membase[memx], 0, memsize[memx]);
  mallco_dev.memrdy[memx] = 1;
}

/**
  * @brief  peruse memory
  * @param  memx: memory block
  * @retval usage rate(0-100)
  */
u8 my_mem_perused(u8 memx)
{
  u32 used = 0;
  u32 i;

  for(i = 0; i < memtblsize[memx]; i++)
  {
    if(mallco_dev.memmap[memx][i])
    {
      used++;
    }
  }

  return (used * 100) / (memtblsize[memx]);
}

/**
  * @brief  malloc memory
  * @param  memx: memory block
  * @param  size: memory size(byte)
  * @retval 0XFFFFFFFF: error, other: memory offset
  */
u32 my_mem_malloc(u8 memx, u32 size)
{
  signed long offset = 0;
  u32 nmemb;
  u32 cmemb = 0;
  u32 i;

  if(!mallco_dev.memrdy[memx])
  {
    mallco_dev.init(memx);
  }

  if(size == 0)
  {
    return 0XFFFFFFFF;
  }

  nmemb = size / memblksize[memx];

  if(size % memblksize[memx])
  {
    nmemb++;
  }

  for(offset = memtblsize[memx] - 1; offset >= 0; offset--)
  {
    if(!mallco_dev.memmap[memx][offset])
    {
      cmemb++;
    }
    else
    {
      cmemb = 0;
    }

    if(cmemb == nmemb)
    {
      for(i = 0; i < nmemb; i++)
      {
        mallco_dev.memmap[memx][offset + i] = nmemb;
      }

      return (offset * memblksize[memx]);
    }
  }

  return 0XFFFFFFFF;
}

/**
  * @brief  free memory
  * @param  memx: memory block
  * @param  offset: memory offset
  * @retval 0: success, 1: failure
  */
u8 my_mem_free(u8 memx, u32 offset)
{
  int i;

  if(!mallco_dev.memrdy[memx])
  {
    mallco_dev.init(memx);
    return 1;
  }

  if(offset < memsize[memx])
  {
    int index = offset / memblksize[memx];
    int nmemb = mallco_dev.memmap[memx][index];

    for(i = 0; i < nmemb; i++)
    {
      mallco_dev.memmap[memx][index + i] = 0;
    }

    return 0;
  }
  else
  {
    return 2;
  }
}

/**
  * @brief  free memory
  * @param  memx: memory block
  * @param  ptr: first address of memory
  * @retval none
  */
void myfree(u8 memx, void *ptr)
{
  u32 offset;

  if(ptr == NULL)
  {
    return;
  }

  offset = (u32)ptr - (u32)mallco_dev.membase[memx];
  my_mem_free(memx, offset);
}

/**
  * @brief  malloc memory
  * @param  memx: memory block
  * @param  size: memory size(byte)
  * @retval first address of malloc memory
  */
void *mymalloc(u8 memx, u32 size)
{
  u32 offset;
  offset = my_mem_malloc(memx, size);

  if(offset == 0XFFFFFFFF)
  {
    return NULL;
  }
  else
  {
    return (void*)((u32)mallco_dev.membase[memx] + offset);
  }
}

/**
  * @brief  remalloc memory
  * @param  memx: memory block
  * @param  ptr: old first address of memory
  * @param  size: memory size(byte)
  * @retval new first address of malloc memory
  */
void *myrealloc(u8 memx, void *ptr, u32 size)
{
  u32 offset;
  offset = my_mem_malloc(memx, size);

  if(offset == 0XFFFFFFFF)
  {
    printf("\r\nrealloc_Err");
    return NULL;
  }

  else
  {
    mymemcpy((void*)((u32)mallco_dev.membase[memx] + offset), ptr, size);
    myfree(memx, ptr);
    return (void*)((u32)mallco_dev.membase[memx] + offset);
  }
}
